//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Input_Library, Example4 - Joystick
//
// Date: 13th January, 2004
// Author(s): Darren John Dwyer, Brian Austin
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This demo shows how to control a force-feedback joystick.
// It enumerates devices, allows you to retrieve a devices capabilities,
// create an interface to a force feedback joystick, create a force-feedback
// effect and send it to the joystick.
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma link "TDx_Input_Library_Install"
#pragma link "TDx_Input"
#pragma link "TDx_InputDevice"
#pragma link "TDIDevCaps"
#pragma link "tdx_inputeffect"
#pragma link "tdidataformat"
#pragma link "tdijoystate2"
#pragma link "tdipropdword"
#pragma link "tdipropheader"
#pragma link "tdiperiodic"
#pragma link "tdienvelope"
#pragma link "tdieffect"
#pragma link "tdidevcaps"
#pragma link "tdx_input"
#pragma link "tdx_input_library_functions"
#pragma link "tdx_inputdevice"
#pragma link "tdiconstantforce"
#pragma link "tdieffectinfo"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Width=780;
Height=580;
Position=poScreenCenter;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (Dx_Input1->Created) Dx_Input1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
if (Dx_Input1->Created) Dx_Input1->Destroy();
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearMessagesButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1EnumDevices(TObject *Sender,
    TDIDeviceInstance *DeviceInstance, void *Context, bool &Finished)
{
if (NumDevices<MAX_DEVICES)
    {
    DevTypes[NumDevices] = DeviceInstance->DevType;
    FFDriverGuids[NumDevices] = DeviceInstance->FFDriver;
    InstanceGuids[NumDevices] = DeviceInstance->Instance;
    InstanceNames[NumDevices] = DeviceInstance->InstanceName;
    ProductGuids[NumDevices] = DeviceInstance->Product;
    ProductNames[NumDevices] = DeviceInstance->ProductName;
    Usages[NumDevices] = DeviceInstance->Usage;
    UsagePages[NumDevices] = DeviceInstance->UsagePage;
    NumDevices++;

    Msg("Found a device: "+DeviceInstance->ProductName);
    DevicesListBox->Items->Add( DeviceInstance->ProductName );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
Msg("ERROR ["+Error+"]: "+Function+", "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1Create(TObject *Sender)
{
Msg("DirectInput Interface has been created");

Msg("Scanning for Available Devices");

NumDevices = 0;
Dx_Input1->EnumDevices(DIDEVTYPE_JOYSTICK, NULL, DIEDFL_FORCEFEEDBACK | DIEDFL_ATTACHEDONLY );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DevicesListBoxClick(TObject *Sender)
{
if (DevicesListBox->Items->Count>0)
    {
    int index = DevicesListBox->ItemIndex;

    AnsiString device_name = DevicesListBox->Items->Strings[index];

	Msg("Creating Selected Device: '"+device_name+"'");

	if (Dx_InputDevice1->Created)
		{
        Msg("Destroying Active Device");
	    Dx_InputDevice1->Destroy();
        }

	Dx_InputDevice1->Create( ProductGuids[DevicesListBox->ItemIndex], Dx_Input1 );
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1Destroy(TObject *Sender)
{
Msg("DirectInput interface is about to be destroyed." );

if (Dx_InputDevice1->Created) Dx_InputDevice1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDevice1Create(TObject *Sender)
{
Msg("Input Device has been Created");

Msg("Retrieving Device Capabilities");
DeviceCapsListBox->Items->Clear();
if (Dx_InputDevice1->GetCapabilities( DIDevCaps1 ))
    {
    DeviceCapsListBox->Items->Add( "Axis: "+IntToStr( (int) DIDevCaps1->Axes) );
    DeviceCapsListBox->Items->Add( "Buttons: "+IntToStr( (int) DIDevCaps1->Buttons) );
    DeviceCapsListBox->Items->Add( "Buttons: "+IntToStr( (int) DIDevCaps1->Buttons) );
    DeviceCapsListBox->Items->Add( "FFDriverVersion: "+IntToStr( (int) DIDevCaps1->FFDriverVersion) );
    DeviceCapsListBox->Items->Add( "FFMinTimeResolution: "+IntToStr( (int) DIDevCaps1->FFMinTimeResolution) );
    DeviceCapsListBox->Items->Add( "FFSamplePeriod: "+IntToStr( (int) DIDevCaps1->FFSamplePeriod) );
    DeviceCapsListBox->Items->Add( "FirmwareRevision: "+IntToStr( (int) DIDevCaps1->FirmwareRevision) );
    DeviceCapsListBox->Items->Add( "Flags:" );
    DeviceCapsListBox->Items->AddStrings( DIDevCaps1->Flags_Strings );
    DeviceCapsListBox->Items->Add( "HardwareRevision: "+IntToStr( (int) DIDevCaps1->HardwareRevision) );
    DeviceCapsListBox->Items->Add( "POV's: "+IntToStr( (int) DIDevCaps1->POVs) );
    }
else
    {
    DeviceCapsListBox->Items->Add( "------ Unknown ------" );
    }

Msg("Enumerating Effects");
NumEffects = 0;
EffectsListBox->Items->Clear();
Dx_InputDevice1->EnumEffects( NULL, DIEFT_ALL );

Msg("Enumerating Objects");
NumObjects = 0;
ObjectsListBox->Items->Clear();
Dx_InputDevice1->EnumObjects( NULL, DIDFT_PSHBUTTON );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDevice1Destroy(TObject *Sender)
{
Msg("Input Device is about to be Destroyed");

Dx_InputEffect1->Destroy();
Dx_InputDevice1->Unacquire();
}
//--------------------------------------------------------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDevice1EnumEffects(TObject *Sender, TDIEffectInfo *EffectInfo, void *Context,
    bool &Finished)
{
if (NumEffects<MAX_EFFECTS)
    {
    EffectTypes[NumEffects] = EffectInfo->EffectType;
    EffectGuids[NumEffects] = EffectInfo->Guid;
    EffectNames[NumEffects] = EffectInfo->Name;
    EffectStaticParams[NumEffects] = EffectInfo->StaticParams;
    EffectDynamicParams[NumEffects] = EffectInfo->DynamicParams;

    EffectsListBox->Items->Add( EffectInfo->Name );

    NumEffects++;
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateEffectButtonClick(TObject *Sender)
{
static dword   axes[2] = { DIJOFS_X, DIJOFS_Y };
static long    direction[3] = { 1, 0, 0 };

switch (EffectDirection->ItemIndex)
	{
    case 0:	// left
    	direction[0] = 1;
        direction[1] = 0;
    	break;
    case 1:	// right
    	direction[0] = -1;
        direction[1] = 0;
    	break;
    case 2:	// up
    	direction[0] = 0;
        direction[1] = 1;
    	break;
    case 3:	// down
    	direction[0] = 0;
        direction[1] = -1;
    	break;
    }

Dx_InputDevice1->SetDataFormat( DIJoyState2 );
Dx_InputDevice1->SetCooperativeLevel( Handle, DISCL_EXCLUSIVE | DISCL_FOREGROUND );
Dx_InputDevice1->Acquire();

// Most values for DIEffect2 have been setup at design time using the BCB Object Inspector.
// The following values should be setup at run time :-

DIEffect2->TypeSpecificParamsSize = DIConstantForce1->Size;
DIEffect2->TypeSpecificParams = DIConstantForce1->Internal_DICONSTANTFORCE_Ptr;

DIEffect2->Duration = INFINITE;

DIEffect2->AxesCount = 2;
DIEffect2->Axes = &axes[0];
DIEffect2->Direction = &direction[0];

if (EffectButton->ItemIndex==11)
	{
    DIEffect2->Flags = DIEFF_CARTESIAN | DIEFF_OBJECTOFFSETS;
    DIEffect2->TriggerButton = DIEB_NOTRIGGER;
    }
else if (EffectButton->ItemIndex>NumObjects-1)
	{
    ShowMessage("Button # is too high");
    EffectButton->ItemIndex = 11;
    }
else
	{
    DIEffect2->Flags = DIEFF_CARTESIAN | DIEFF_OBJECTOFFSETS;
    DIEffect2->TriggerButton = EffectButton->ItemIndex;
    }


if (!Dx_InputEffect1->Create( GUID_ConstantForce, DIEffect2, Dx_InputDevice1 ))
	{
    Dx_InputDevice1->Unacquire();
    }

/*
DIPropHeader1->Obj = 0;
DIPropHeader1->How = DIPH_DEVICE;
DIPropDWORD1->PropHeader = DIPropHeader1;
DIPropDWORD1->Data = 0;

Dx_InputDevice1->SetProperty( DIPROP_AUTOCENTER, DIPropDWORD1 );

DIPeriodic1->Magnitude = DI_FFNOMINALMAX;
DIPeriodic1->Offset = 0;
DIPeriodic1->Phase = 0;
DIPeriodic1->Period = (DWORD) (0.05 * DI_SECONDS);
DIEnvelope1->AttackLevel = 0;
DIEnvelope1->AttackTime = (DWORD) (0.5 * DI_SECONDS);
DIEnvelope1->FadeLevel = 0;
DIEnvelope1->FadeTime = (DWORD) (1.0 * DI_SECONDS);

DIEffect1->Flags = DIEFF_POLAR | DIEFF_OBJECTOFFSETS;
DIEffect1->Duration = (DWORD) (2 * DI_SECONDS);

DIEffect1->SamplePeriod = 0;               // = default
DIEffect1->Gain = DI_FFNOMINALMAX;         // no scaling
DIEffect1->TriggerButton = DIJOFS_BUTTON0;
DIEffect1->TriggerRepeatInterval = 0;
DIEffect1->AxesCount = 2;
DIEffect1->Axes = &axes[0];
DIEffect1->Direction = &direction[0];
DIEffect1->Envelope = DIEnvelope1;
DIEffect1->TypeSpecificParams = DIPeriodic1->Internal_DIPERIODIC_Ptr;
DIEffect1->TypeSpecificParamsSize = DIPeriodic1->Size;

Dx_InputEffect1->Create( EffectGuids[EffectsListBox->ItemIndex], DIEffect1, Dx_InputDevice1 );
*/
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StopEffectButtonClick(TObject *Sender)
{
Dx_InputEffect1->Stop();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PlayEffectButtonClick(TObject *Sender)
{
Dx_InputEffect1->Start( 1, 0 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyEffectButtonClick(TObject *Sender)
{
Dx_InputEffect1->Destroy();
Dx_InputDevice1->Unacquire();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
Msg("Creating DirectInput Interface");

if (Dx_Input1->Created)
    Dx_Input1->Destroy();

Dx_Input1->Create();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDevice1EnumObjects(TObject *Sender,
	TDIDeviceObjectInstance *ObjectInstance, void *Context, bool &Finished)
{
if (NumObjects<MAX_OBJECTS)
	{
    ObjectNames[NumObjects] = ObjectInstance->Name;
    ObjectIDs[NumObjects] = DIDFT_GETINSTANCE( ObjectInstance->Flags );
    ObjectOffsets[NumObjects] = ObjectInstance->Ofs;

    ObjectsListBox->Items->Add( ObjectInstance->Name );
    
    NumObjects++;
    }
else
	Finished = true;
}
//---------------------------------------------------------------------------
